@@ -20,6 +20,7 @@ from typing import Callable, NamedTuple, Optional, Dict, Sequence
 _parameter_break = re.compile("\n(?=[A-Za-z_])")
 _section_break = re.compile(r"\n(?=[^\n]{3,15}\n-{3,15})", re.MULTILINE)
 _numpy_signature_re = re.compile(r'^([\w., ]+=)?\s*[\w\.]+\([\w\W]*?\)$', re.MULTILINE)
+_versionadded = re.compile(r'^\s+\.\.\s+versionadded::', re.MULTILINE)
 
 class ParsedDoc(NamedTuple):
   """
@@ -85,7 +86,8 @@ def _parse_parameters(body: str) -> Dict[str, str]:
 
 
 def _wraps(fun: Callable, update_doc: bool = True, lax_description: str = "",
-           sections: Sequence[str] = ('Parameters', 'Returns', 'References')):
+           sections: Sequence[str] = ('Parameters', 'Returns', 'References'),
+           skip_params: Sequence[str] = ()):
   """Specialized version of functools.wraps for wrapping numpy functions.
 
   This produces a wrapped function with a modified docstring. In particular, if
@@ -103,6 +105,8 @@ def _wraps(fun: Callable, update_doc: bool = True, lax_description: str = "",
       the docstring.
     sections: a list of sections to include in the docstring. The default is
       ["Parameters", "returns", "References"]
+    skip_params: a list of strings containing names of parameters accepted by the
+      function that should be skipped in the parameter list.
   """
   def wrap(op):
     docstr = getattr(fun, "__doc__", None)
@@ -116,8 +120,8 @@ def _wraps(fun: Callable, update_doc: bool = True, lax_description: str = "",
           parsed.sections['Parameters'] = (
             "Parameters\n"
             "----------\n" +
-            "\n".join(desc for p, desc in parameters.items()
-                      if p in op.__code__.co_varnames)
+            "\n".join(_versionadded.split(desc)[0].rstrip() for p, desc in parameters.items()
+                      if p in op.__code__.co_varnames and p not in skip_params)
           )
 
         docstr = parsed.summary.strip() + "\n" if parsed.summary else ""
