@@ -19,8 +19,9 @@ import numpy as np
 
 from . import core
 from . import dtypes
-from .tree_util import (tree_flatten, tree_unflatten, tree_multimap, _replace_nones,
+from .tree_util import (tree_flatten, tree_unflatten, tree_multimap,
                         tree_structure, treedef_children, treedef_is_leaf)
+from ._src.tree_util import _replace_nones
 from . import linear_util as lu
 from ._src.util import safe_map, WrapHashably, Hashable
 from .core import unit
@@ -216,3 +217,8 @@ def shaped_abstractify(x):
   named_shape = getattr(x, 'named_shape', {})
   return core.ShapedArray(np.shape(x), _dtype(x), weak_type=weak_type,
                           named_shape=named_shape)
+
+# This decorator exists to make it easier to monkey-patch APIs in JAX.
+# By default it does nothing, but it can be monkey-patched to do other things.
+def api_hook(fun, tag: str):
+  return fun
