@@ -12,11 +12,16 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+# TODO(phawkins): this file triggers a pytype bug.
+# pytype: skip-file
+
 import contextlib
 import functools
 import os
 import sys
 import threading
+from typing import List, Callable, Optional
+import warnings
 
 from jax import lib
 
@@ -51,13 +56,12 @@ class Config:
   def __init__(self):
     self.values = {}
     self.meta = {}
-    self.FLAGS = NameSpace(self.read)
+    self.FLAGS = NameSpace(self.read, self.update)
     self.use_absl = False
     self._contextmanager_flags = set()
+    self._update_hooks = {}
 
-    # TODO(mattjj): delete these when only omnistaging is available
-    self.omnistaging_enabled = bool_env('JAX_OMNISTAGING', True)
-    self._omnistaging_disablers = []
+    self.omnistaging_enabled = True  # TODO(mattjj): remove this
 
   def update(self, name, val):
     if self.use_absl:
@@ -68,10 +72,9 @@ class Config:
         raise Exception("Unrecognized config option: {}".format(name))
       self.values[name] = val
 
-    if name == "jax_disable_jit":
-      lib.jax_jit.global_state().disable_jit = val
-    elif name == "jax_enable_x64":
-      lib.jax_jit.global_state().enable_x64 = val
+    hook = self._update_hooks.get(name, None)
+    if hook:
+      hook(val)
 
   def read(self, name):
     if name in self._contextmanager_flags:
@@ -87,27 +90,36 @@ class Config:
       self.check_exists(name)
       return self.values[name]
 
-  def add_option(self, name, default, opt_type, meta_args, meta_kwargs):
+  def add_option(self, name, default, opt_type, meta_args, meta_kwargs,
+                 update_hook=None):
     if name in self.values:
       raise Exception("Config option {} already defined".format(name))
     self.values[name] = default
     self.meta[name] = (opt_type, meta_args, meta_kwargs)
+    if update_hook:
+      self._update_hooks[name] = update_hook
+      update_hook(default)
 
   def check_exists(self, name):
     if name not in self.values:
       raise AttributeError("Unrecognized config option: {}".format(name))
 
   def DEFINE_bool(self, name, default, *args, **kwargs):
-    self.add_option(name, default, bool, args, kwargs)
+    update_hook = kwargs.pop("update_hook", None)
+    self.add_option(name, default, bool, args, kwargs, update_hook=update_hook)
 
   def DEFINE_integer(self, name, default, *args, **kwargs):
-    self.add_option(name, default, int, args, kwargs)
+    update_hook = kwargs.pop("update_hook", None)
+    self.add_option(name, default, int, args, kwargs, update_hook=update_hook)
 
   def DEFINE_string(self, name, default, *args, **kwargs):
-    self.add_option(name, default, str, args, kwargs)
+    update_hook = kwargs.pop("update_hook", None)
+    self.add_option(name, default, str, args, kwargs, update_hook=update_hook)
 
   def DEFINE_enum(self, name, default, *args, **kwargs):
-    self.add_option(name, default, 'enum', args, kwargs)
+    update_hook = kwargs.pop("update_hook", None)
+    self.add_option(name, default, 'enum', args, kwargs,
+                    update_hook=update_hook)
 
   def config_with_absl(self):
     # Run this before calling `app.run(main)` etc
@@ -141,33 +153,25 @@ class Config:
       already_configured_with_absl = True
 
       if not FLAGS.jax_omnistaging:
-        self.disable_omnistaging()
-
-  def register_omnistaging_disabler(self, disabler):
-    if self.omnistaging_enabled:
-      self._omnistaging_disablers.append(disabler)
-    else:
-      disabler()
+        raise Exception(
+          "Disabling of omnistaging is no longer supported in JAX version 0.2.12 and higher: "
+          "see https://github.com/google/jax/blob/master/design_notes/omnistaging.md.\n"
+          "To remove this warning, unset the JAX_OMNISTAGING environment variable.")
 
   def enable_omnistaging(self):
-    if not self.omnistaging_enabled:
-      raise Exception("can't re-enable omnistaging after it's been disabled")
+    warnings.warn(
+        "enable_omnistaging() is a no-op in JAX versions 0.2.12 and higher;\n"
+        "see https://github.com/google/jax/blob/master/design_notes/omnistaging.md")
 
   def disable_omnistaging(self):
-    if self.omnistaging_enabled:
-      for disabler in self._omnistaging_disablers:
-        disabler()
-      self.omnistaging_enabled = False
+    raise Exception(
+      "Disabling of omnistaging is no longer supported in JAX version 0.2.12 and higher: "
+      "see https://github.com/google/jax/blob/master/design_notes/omnistaging.md.")
 
-#   # TODO(jakevdp, mattjj): unify this with `define_bool_state` stuff below
-#   @property
-#   def x64_enabled(self):
-#     return lib.jax_jit.get_enable_x64()
-
-#   def _set_x64_enabled(self, state):
-#     lib.jax_jit.thread_local_state().enable_x64 = bool(state)
-
-  def define_bool_state(self, name: str, default: bool, help: str):
+  def define_bool_state(
+    self, name: str, default: bool, help: str, *,
+    update_global_hook: Optional[Callable[[bool], None]] = None,
+    update_thread_local_hook: Optional[Callable[[Optional[bool]], None]] = None):
     """Set up thread-local state and return a contextmanager for managing it.
 
     This function is a convenience wrapper. It defines a flag and corresponding
@@ -183,6 +187,11 @@ class Config:
       default: boolean, a default value for the option.
       help: string, used to populate the flag help information as well as the
         docstring of the returned context manager.
+      update_global_hook: a optional callback that is called with the updated
+        value of the global state when it is altered or set initially.
+      update_thread_local_hook: a optional callback that is called with the
+        updated value of the thread-local state when it is altered or set
+        initially.
 
     Returns:
       A contextmanager to control the thread-local state value.
@@ -208,7 +217,36 @@ class Config:
     an error.
     """
     name = name.lower()
-    self.DEFINE_bool(name, bool_env(name.upper(), default), help)
+    self.DEFINE_bool(name, bool_env(name.upper(), default), help,
+                     update_hook=update_global_hook)
+    self._contextmanager_flags.add(name)
+
+    def get_state(self):
+      val = getattr(_thread_local_state, name, unset)
+      return val if val is not unset else self._read(name)
+    setattr(Config, name, property(get_state))
+
+    return _BoolStateContextManager(name, help, update_thread_local_hook)
+
+  def define_enum_state(self, name: str, enum_values: List[str],
+                        default: Optional[str], help: str):
+    """Set up thread-local state and return a contextmanager for managing it.
+    Args:
+      name: string, converted to lowercase to define the name of the config
+        option (and absl flag). It is converted to uppercase to define the
+        corresponding shell environment variable.
+      enum_values: list of strings representing the possible values for the
+        option.
+      default: optional string, default value.
+      help: string, used to populate the flag help information as well as the
+        docstring of the returned context manager.
+    Returns:
+      A contextmanager to control the thread-local state value.
+    See docstring for ``define_bool_state``.
+    """
+    name = name.lower()
+    self.DEFINE_enum(name, os.getenv(name.upper(), default),
+                     enum_values=enum_values, help=help)
     self._contextmanager_flags.add(name)
 
     def get_state(self):
@@ -217,7 +255,11 @@ class Config:
     setattr(Config, name, property(get_state))
 
     @contextlib.contextmanager
-    def set_state(new_val: bool):
+    def set_state(new_val: Optional[str]):
+      if (new_val is not None and
+          (type(new_val) is not str or new_val not in enum_values)):
+        raise ValueError(f"new enum value must be None or in {enum_values}, "
+                         f"got {new_val} of type {type(new_val)}.")
       prev_val = getattr(_thread_local_state, name, unset)
       setattr(_thread_local_state, name, new_val)
       try:
@@ -231,18 +273,55 @@ class Config:
     set_state.__doc__ = f"Context manager for `{name}` config option.\n\n{help}"
     return set_state
 
+class _BoolStateContextManager:
+  def __init__(self, name, help, update_thread_local_hook):
+    self._name = name
+    self.__name__ = name[4:] if name.startswith('jax_') else name
+    self.__doc__ = f"Context manager for `{name}` config option.\n\n{help}"
+    self._hook = update_thread_local_hook
+
+  @contextlib.contextmanager
+  def __call__(self, new_val: bool):
+    prev_val = getattr(_thread_local_state, self._name, unset)
+    setattr(_thread_local_state, self._name, new_val)
+    if self._hook is not None: self._hook(new_val)
+    try:
+      yield
+    finally:
+      if prev_val is unset:
+        delattr(_thread_local_state, self._name)
+        if self._hook is not None: self._hook(None)
+      else:
+        setattr(_thread_local_state, self._name, prev_val)
+        if self._hook is not None: self._hook(prev_val)
+
+  def _add_hooks(self, update_global_hook, update_thread_local_hook):
+    """Private method that adds hooks to an existing context-manager.
+
+    Used to avoid cyclic import dependencies."""
+    self._hook = update_thread_local_hook
+    config._update_hooks[self._name] = update_global_hook
+    update_global_hook(config._read(self._name))
+
+
 _thread_local_state = threading.local()
 
-class Unset: pass
-unset = Unset()
+class _Unset: pass
+unset = _Unset()
 
-class NameSpace(object):
-  def __init__(self, getter):
-    self._getter = getter
+class NameSpace:
+  def __init__(self, getter, setter):
+    # must use super because we override this class's __setattr__, see
+    # https://docs.python.org/3/reference/datamodel.html#object.__setattr__
+    super().__setattr__('_getter', getter)
+    super().__setattr__('_setter', setter)
 
   def __getattr__(self, name):
     return self._getter(name)
 
+  def __setattr__(self, name, val):
+    self._setter(name, val)
+
 
 config = Config()
 flags = config
@@ -251,10 +330,12 @@ FLAGS = flags.FLAGS
 already_configured_with_absl = False
 
 
+# TODO(mattjj): remove all uses of this flag
 flags.DEFINE_bool(
     'jax_omnistaging',
     bool_env('JAX_OMNISTAGING', True),
-    help='Enable staging based on dynamic context rather than data dependence.'
+    help=('Deprecated. Setting this flag to False raises an error. Setting it '
+          'to True has no effect.'),
 )
 
 flags.DEFINE_integer(
@@ -316,33 +397,62 @@ log_compiles = config.define_bool_state(
           'option is set, the log level is WARNING; otherwise the level is '
           'DEBUG.'))
 
-# Because jax_enable_x64 is managed by C++ code, we don't reuse the
-# config.define_bool_state mechanism, though conceptually it is the same.
-config.DEFINE_bool('jax_enable_x64', bool_env('JAX_ENABLE_X64', False),
-                   help='Enable 64-bit types to be used')
-lib.jax_jit.global_state().enable_x64 = bool_env('JAX_ENABLE_X64', False)
+def _update_x64_global(val):
+  lib.jax_jit.global_state().enable_x64 = val
 
-@contextlib.contextmanager
-def enable_x64(new_val: bool = True):
-  """Experimental context manager to temporarily enable X64 mode.
+def _update_x64_thread_local(val):
+  lib.jax_jit.thread_local_state().enable_x64 = val
 
-  Usage::
+enable_x64 = config.define_bool_state(
+    name='jax_enable_x64',
+    default=False,
+    help='Enable 64-bit types to be used',
+    update_global_hook=_update_x64_global,
+    update_thread_local_hook=_update_x64_thread_local)
 
-    >>> import jax.numpy as jnp
-    >>> with enable_x64(True):
-    ...   print(jnp.arange(10.0).dtype)
-    ...
-    float64
-  """
-  prev_val = config.jax_enable_x64
-  lib.jax_jit.thread_local_state().enable_x64 = bool(new_val)
-  try:
-    yield
-  finally:
-    lib.jax_jit.thread_local_state().enable_x64 = prev_val
-Config.jax_enable_x64 = property(lambda self: lib.jax_jit.get_enable_x64())
-# config._contextmanager_flags.add('jax_enable_x64')  # TODO(mattjj): remove footgun
+# TODO(phawkins): remove after fixing users of FLAGS.x64_enabled.
+config._contextmanager_flags.remove("jax_enable_x64")
 
-# The `x64_enabled` property doesn't fit the naming scheme, but we use it for
-# backward compatibility.
-Config.x64_enabled = Config.jax_enable_x64
+Config.x64_enabled = Config.jax_enable_x64  # type: ignore
+
+def _update_disable_jit_global(val):
+  lib.jax_jit.global_state().disable_jit = val
+
+def _update_disable_jit_thread_local(val):
+  lib.jax_jit.thread_local_state().disable_jit = val
+
+disable_jit = config.define_bool_state(
+    name='jax_disable_jit',
+    default=False,
+    help=('Disable JIT compilation and just call original Python.'),
+    update_global_hook=_update_disable_jit_global,
+    update_thread_local_hook=_update_disable_jit_thread_local)
+
+
+numpy_rank_promotion = config.define_enum_state(
+    name='jax_numpy_rank_promotion',
+    enum_values=['allow', 'warn', 'raise'],
+    default='allow',
+    help=('Control NumPy-style automatic rank promotion broadcasting '
+          '("allow", "warn", or "raise").'))
+
+default_matmul_precision = config.define_enum_state(
+    name='jax_default_matmul_precision',
+    enum_values=['bfloat16', 'tensorfloat32', 'float32'],
+    default=None,
+    help=('Control the default matmul and conv precision for 32bit inputs.\n\n'
+
+          'Some platforms, like TPU, offer configurable precision levels for '
+          'matrix multiplication and convolution computations, trading off '
+          'accuracy for speed. The precision can be controlled for each '
+          'operation; for example, see the :func:`jax.lax.conv_general_dilated` '
+          'and :func:`jax.lax.dot` docstrings. But it can be useful to control '
+          'the default behavior obtained when an operation is not given a '
+          'specific precision.\n\n'
+
+          'This option can be used to control the default precision '
+          'level for computations involved in matrix multiplication and '
+          'convolution on 32bit inputs. The levels roughly describe the '
+          "precision at which scalar products are computed. The 'bfloat16' "
+          "option is the fastest and least precise; 'float32' is similar to "
+          "full float32 precision; 'tensorfloat32' is intermediate.\n\n"))
