@@ -1819,166 +1819,6 @@ def pp_kv_pairs(kv_pairs):
   else:
     return pp('')
 
-@config.register_omnistaging_disabler
-def omnistaging_disabler() -> None:
-  global thread_local_state, call_bind, find_top_trace, initial_style_staging, \
-      new_main, reset_trace_state, TraceStack, TraceState, extend_axis_env, \
-      eval_context, extra_jit_context
-
-  class TraceStack:
-    upward: List[MainTrace]
-    downward: List[MainTrace]
-
-    def __init__(self):
-      self.upward = []
-      self.downward = []
-
-    def next_level(self, bottom: bool) -> int:
-      if bottom:
-        return - (len(self.downward) + 1)
-      else:
-        return len(self.upward)
-
-    def push(self, main_trace: MainTrace, bottom: bool) -> None:
-      if bottom:
-        self.downward.append(main_trace)
-      else:
-        self.upward.append(main_trace)
-
-    def pop(self, bottom: bool) -> None:
-      if bottom:
-        self.downward.pop()
-      else:
-        self.upward.pop()
-
-    def __repr__(self) -> str:
-      return  'Trace stack\n{} ---\n{}'.format(
-        map('  {}\n'.format, self.upward[::-1]),
-        map('  {}\n'.format, self.downward))
-
-    def copy(self):
-      new = TraceStack()
-      new.upward = self.upward[:]
-      new.downward = self.downward[:]
-      return new
-
-  class TraceState:
-    trace_stack: TraceStack
-    substack: List[Sublevel]
-    initial_style: bool
-
-    def __init__(self) -> None:
-      self.trace_stack = TraceStack()  # type: ignore
-      self.substack = [Sublevel(0)]
-      self.initial_style = False
-
-    def copy(self):
-      new = TraceState()
-      new.trace_stack = self.trace_stack.copy()
-      new.substack = self.substack[:]
-      new.initial_style = self.initial_style
-      return new
-
-  def extra_jit_context(trace_stack):
-    return None
-
-  thread_local_state = ThreadLocalState()
-
-  def reset_trace_state() -> bool:
-    "Reset the global trace state and return True if it was already clean."
-    if (thread_local_state.trace_state.substack != [Sublevel(0)] or
-        thread_local_state.trace_state.trace_stack.downward or
-        thread_local_state.trace_state.trace_stack.upward):
-      thread_local_state.trace_state.__init__()  # type: ignore
-      return False
-    else:
-      return True
-
-  @contextmanager
-  def new_main(trace_type: Type[Trace], bottom=False, **payload) -> Generator[MainTrace, None, None]:
-    level = thread_local_state.trace_state.trace_stack.next_level(bottom)
-    main = MainTrace(level, trace_type, **payload)
-    thread_local_state.trace_state.trace_stack.push(main, bottom)
-
-    try:
-      yield main
-    finally:
-      thread_local_state.trace_state.trace_stack.pop(bottom)
-
-    if config.jax_check_tracer_leaks:
-      t = ref(main)
-      del main
-      if t() is not None:
-        print(thread_local_state.trace_state.trace_stack)
-        raise Exception('Leaked trace {}'.format(t()))
-
-  def find_top_trace(xs) -> Optional[Trace]:
-    top_trace = max((x._trace for x in xs if isinstance(x, Tracer)),
-                    key=attrgetter('level'), default=None)
-    return top_trace and top_trace.main.with_cur_sublevel()
-
-  @contextmanager
-  def eval_context():
-    yield  # dummy implementation for forward compatibility
-
-  def bind(self, *args, **kwargs):
-    assert not config.jax_enable_checks or all(isinstance(arg, Tracer)
-                              or valid_jaxtype(arg) for arg in args), args
-    top_trace = find_top_trace(args)
-    if top_trace is None:
-      return self.impl(*args, **kwargs)
-
-    tracers = map(top_trace.full_raise, args)
-    out_tracer = top_trace.process_primitive(self, tracers, kwargs)
-    if self.multiple_results:
-      return map(full_lower, out_tracer)
-    else:
-      return full_lower(out_tracer)
-  Primitive.bind = bind  # type: ignore
-
-  def call_bind(primitive: Union['CallPrimitive', 'MapPrimitive'],
-                fun: lu.WrappedFun, *args, **params):
-    out_axes_transforms = _IgnoreElemList()
-    if primitive.map_primitive:
-      out_axes_thunk = params['out_axes_thunk']
-      # The new thunk depends deterministically on the old thunk and the wrapped function.
-      # Any caching already has to include the wrapped function as part of the key, so we
-      # only use the previous thunk for equality checks.
-      @as_hashable_function(closure=out_axes_thunk)
-      def new_out_axes_thunk():
-        out_axes = out_axes_thunk()
-        for t in out_axes_transforms:
-          out_axes = t(out_axes)
-        return out_axes
-      params = dict(params, out_axes_thunk=new_out_axes_thunk)
-    params_tuple = tuple(params.items())
-    top_trace = find_top_trace(args)
-    level = (thread_local_state.trace_state.trace_stack.next_level(True)
-            if top_trace is None else top_trace.level)
-    params_tuple = tuple(params.items())
-    fun, env_trace_todo = process_env_traces(
-        fun, primitive, level, params_tuple, out_axes_transforms)
-    if top_trace is None:
-      with new_sublevel():
-        outs = primitive.impl(fun, *args, **params)
-    else:
-      tracers = map(top_trace.full_raise, args)
-      outs = primitive.process(top_trace, fun, tracers, params)
-    return apply_todos(env_trace_todo(), map(full_lower, outs))
-
-  @contextmanager
-  def extend_axis_env(axis_name, size: int, tag: Any):
-    yield
-
-  @contextmanager
-  def initial_style_staging():
-    trace_state = thread_local_state.trace_state
-    prev, trace_state.initial_style = trace_state.initial_style, True
-    try:
-      yield
-    finally:
-      trace_state.initial_style = prev
-
 # Casting float0 array to a float-valued zero array.
 def zeros_like_float0(array, dtype=None):
   if not dtype:
