@@ -89,48 +89,3 @@ try:
   from jaxlib import tpu_client  # pytype: disable=import-error
 except:
   tpu_client = None
-
-
-# Backwards compatibility shims for the C++ jax_jit.global_state and
-# jax_jit.thread_local_state which were introduced in XLA extension version 9
-# (jaxlib 0.1.64).
-if _xla_extension_version <= 8:
-  class GlobalJitState:
-    @property
-    def disable_jit(self):
-      return jax_jit.get_disable_jit_cpp_flag()
-
-    @disable_jit.setter
-    def disable_jit(self, x):
-      jax_jit.set_disable_jit_cpp_flag(x)
-
-    @property
-    def enable_x64(self):
-      return jax_jit.get_enable_x64_cpp_flag()
-
-    @enable_x64.setter
-    def enable_x64(self, x):
-      jax_jit.set_enable_x64_cpp_flag(x)
-
-  global_state = GlobalJitState()
-  jax_jit.global_state = lambda: global_state
-
-
-  class ThreadLocalJitState:
-    @property
-    def disable_jit(self):
-      return jax_jit.get_disable_jit_thread_local()
-
-    @disable_jit.setter
-    def disable_jit(self, x):
-      jax_jit.set_disable_jit_thread_local(x)
-
-    @property
-    def enable_x64(self):
-      return jax_jit.get_enable_x64_thread_local()
-
-    @enable_x64.setter
-    def enable_x64(self, x):
-      jax_jit.set_enable_x64_thread_local(x)
-  thread_local_state = ThreadLocalJitState()
-  jax_jit.thread_local_state = lambda: thread_local_state
